@@ -1,7 +1,7 @@
 
 /*  $Header$
  *
- *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -372,6 +372,12 @@ struct button buttons[NKEYS] = {
     do_immed, 
     FUNC
 },
+{
+    KEY_LOGARITHM2,
+    "Log2",
+    do_immed, 
+    FUNC
+},
 {
     KEY_ABSOLUTE_VALUE,
     "Abs",
@@ -756,7 +762,10 @@ main(int argc, char **argv)
     show_display(v->MPdisp_val);     /* Output in correct display mode. */
 
     memset(&(v->h), 0, sizeof(struct exprm_state_history)); /* clear expression mode state history*/
-
+    struct exprm_state *e;
+    e = get_state();
+    e->clear = 1;              /* Clear initial state as if clear was pressed */
+    
     ui_start();                    /* Display the calculator. */
     
     return(0);
